-- Creates a new enemy at the enemy table
function enemy.create(enemyID, posX, posY, width, height, rotation, AIMove_Type, AIAction_Type, AIAction_Delay, AIParameter1, AIParameter2, AIParameter3, AIParameter4, health)
	-- enemy atributes so far:
	-- posX, posY, spX, spY, AIMove_ReactionTime, AIAction_ReactionTime, AIMove_Chance, AIAction_Chance, enemyImage)
	-- spX and spY are speed in those coordinates.
	-- AIMove_ReactionTime and AIAction_ReactionTime is the time interval where the AI will chech the Move or Action status
	-- AIMove_Chance and AIAction_Chance is the chance to change the moving or action status when ReactionTime happens.
	
	if posX == nil then posX = screenWidth + 50 end
	
	table.insert ( enemy, {enemyID = enemyID, posX = posX, posY = posY, width = width, height = height, rotation = rotation, AIMove_Type = AIMove_Type, AIAction_Type = AIAction_Type, AIAction_Delay = AIAction_Delay, enemyImage = enemyTempImage, AIParameter1 = AIParameter1, AIParameter2 = AIParameter2, AIParameter3 = AIParameter3, AIParameter4 = AIParameter4, health = health } )
	totalObjects = totalObjects + 1
end

-- Removes an enemy element from enemy table
function enemy.remove(enemyID)
	for i, v in ipairs(enemy) do
		if v.enemyID == enemyID then
			table.remove(enemy, i)
			totalObjects = totalObjects - 1
		end
	end
end

function enemy.loadImage(imageName)
	enemyTempImage = love.graphics.newImage("data/sprites/" .. imageName .. ".png")
end

function enemy.damage(k, damage)
	k.health = k.health - damage
	bullet.remove(i)
	if k.health <= 0 then
		enemy.remove(k.enemyID)
	end
end

-- Moves enemies, checks screenBorders and collissions. Usually called from love.update()
function enemy.move()
	for i,v in ipairs(enemy) do
		-- check movement for every enemy every frame
		-- here we will do nothing but call enemy movement types functions
		-- Movement will be "state based". Each frame we check the enemy movement
		-- and reasign the coordinates accordingly to the type of movement
		-- 1 = linear movement
		-- 2 & -2 = diagonal bouncing
		if v.AIMove_Type == 0 then
			-- this movement is for decoration. 
			-- matches movement speed with background
			-- scrolling speed
			staticMovement(v)
		elseif v.AIMove_Type == 1 then
			linearMovement(v)
		elseif v.AIMove_Type == 2 or v.AIMove_Type == -2 then
			zigZagMovement(v)
		elseif v.AIMove_Type == 3 then
			linearTurnMovement(v)
		elseif v.AIMove_Type == 4 then
			keepInRear(v)
		elseif v.AIMove_Type == 5 then
			fullTrackPlayer(v)
		elseif v.AIMove_Type == 6 then
			rearTrackPlayer(v)
		end
	end
end

-- draws all enemies to screen. Usually called from love.draw()
function enemy.draw()
	for i,v in ipairs(enemy) do
		love.graphics.draw(v.enemyImage, v.posX, v.posY, v.rotation)
	end
end


-- USED ONLY FOR TESTING/PROGRAMMING
function enemy.linearMove()
	for i,v in ipairs(enemy) do
		v.posX = v.posX - 5
		
		if v.posX + v.width <= 0 then 
			enemy.remove(v.enemyID)
		end
	end
end

-- ##########################################################
-- 					MOVEMENT ALGORITHMS
-- ##########################################################

function linearMovement(v)
	-- this is the most basic movement type
	-- Only check if the enemy is beyond the left screen edge
	-- and remove it.
	-- Parameter1 : speed
	
	v.posX = v.posX - v.AIParameter1
	
	if v.posX + v.width <= 0 then enemy.remove(v.enemyID) end
end

function zigZagMovement(v)
	-- this is quite basic.
	-- it is not an arc movement so it doesn't
	-- have Y Axis deceleration
	-- Param 4 = radius
	-- Param 2 & 3 = speeds
	-- Param 1 = initial Y position / 2
	-- Dont use this for big ships since this algorithm
	-- is pretty simple and big ships will look bad
	
	local arcRadius = v.AIParameter4
	
	
	if v.AIParameter1 == nil then v.AIParameter1 = v.posY end
	
	nextX = v.posX - v.AIParameter2
	nextY = v.posY - (v.AIParameter3) * v.AIMove_Type/2
	
	if v.AIMove_Type > 0 then
		if math.abs(nextY) <= math.abs(v.AIParameter1 - arcRadius) then
			if v.rotation >= -1*pi/4 then 
				v.rotation = v.rotation - 0.20
				nextY = v.posY - math.abs(100*v.rotation)/100
				nextX = v.posX - 6
			else
				v.AIMove_Type = -2
			end
		end
	else
		if math.abs(nextY) >= math.abs(v.AIParameter1 + arcRadius) then
			if v.rotation <= 1*pi/4 then 
				v.rotation = v.rotation + 0.20
				nextY = v.posY - math.abs(100*v.rotation)/100
				nextX = v.posX - 6
			else
				v.AIMove_Type = 2
			end
		end
	end
	
	v.posX = nextX
	v.posY = nextY
end

function staticMovement(v)
	-- Only used for "static" scenery items
	-- basically it's like linear movement but uses background scroll speed
	-- matches speed with that in the upper background
	-- uses parameter 1 to know if will scroll with upper background
	-- or with middle background.
	-- The only problem with this is to make the top background
	-- to be drawn over this ovject.
	-- But BTW lets keep things simple.
	
	if v.AIParameter1 == 0 or v.AIParameter1 == nil then
		-- set to default speed if not parameter specified.
		v.posX = v.posX - scrollSpeed
	else
		v.posX = v.posX - (scrollSpeed + depthSpeed)
	end
	
	if v.posX + v.width <= 0 then enemy.remove(v.enemyID) end
end

function linearTurnMovement(v)
	-- this movement makes the enemy to go linear towards a destiny spot
	-- and then go back to start
	-- Only used parameter 1 for speed, parameter 2 traveling length and
	-- parameter3 por returning speed
	-- NOTE: AIParameter2 is the X coordinate to go, NOT distance to run
	-- IMPORTANT: I NEED CODE SOMETHING TO MAKE THE ENEMY TO LOOK
	-- OPPOSITE AND MAKE THIS "TURNING" SMOOTH, NOT JUST AN INSTANT TURNING.
	
	v.posX = v.posX - v.AIParameter1
	
	if v.posX <= v.AIParameter2 then
		-- Rotate sprite when reachs end of route
		if v.rotation < pi then
			v.rotation = v.rotation + 0.10
			v.posY = v.posY - 1
			v.AIParameter1 = 0
		else
			v.AIParameter1 = v.AIParameter3
			v.AIParameter1 = -1 * v.AIParameter1
		end
	end
	
	if v.posX + v.width <= -100 or v.posX > screenWidth + v.width + 100 then
		enemy.remove(v.enemyID)
	end
	
end

function keepInRear(v)
	-- Moves to somewhere close to the right screen edge
	-- and stays there shooting.
	-- Parameter1 : Y Axis movement speed. + or - for starting direction, and 0 or nil for no movement
	-- Parameter2 : distance to screen edge. Optional. will be "screenBorder"
	-- if nil or = 0. Cn specify negative movement for making the enemy to partyally show only.
	
	if v.AIParameter1 == nil then v.AIParameter1 = 0 end
	if v.AIParameter2 == nil or v.AIParameter2 <= 0 then 
		v.AIParameter2 = screenWidth - v.width - screenBorder
	end
	
	if v.posX >= v.AIParameter2 then v.posX = v.posX - v.AIParameter1 end
	
	if v.posX <= v.AIParameter2 then
		if v.posY <= screenBorder or v.posY + v.height >= screenHeight - screenBorder  then
			v.AIParameter1 = -1*v.AIParameter1
		end
		
		v.posY = v.posY + v.AIParameter1
	end
end

function fullTrackPlayer(v)
	-- tracks the player in X and Y axis
	-- Parameter1 : speed
	
	-- This could be a bit complex because we will changing angles when the enemy faces the player
	
	-- Store the original speed
	v.AIParameter3 = v.AIParameter1
	
	-- Check player possition against enemy position
	-- Change angle if necessary will be implemented later on
	
	-- X Axis
	if v.posX  > playerPosX  then v.posX = v.posX - v.AIParameter1	end
	if v.posX < playerPosX then v.posX = v.posX + v.AIParameter1 end
	
	-- Y Axis
	if v.posY  > playerPosY then v.posY = v.posY - v.AIParameter1 end
	if v.posY  < playerPosY   then v.posY = v.posY + v.AIParameter1 end
		
end

function rearTrackPlayer(v)
	-- this is a mix between fullTrackPlayer and rear movement
	-- the enemy will keep in rear but will track player in Y axis
	-- Parameters are the same as in rearMovement, but instead making the
	-- enemy freely move in Y Axis, just tracks player
	-- Parameter1 : speed
	-- Parameter2 : goto location
	
	if v.AIParameter1 == nil then v.AIParameter1 = 0 end
	if v.AIParameter2 == nil or v.AIParameter2 <= 0 then 
		v.AIParameter2 = screenWidth - v.width - screenBorder
	end
	
	if v.posX >= v.AIParameter2 then v.posX = v.posX - v.AIParameter1 end
	
	-- BTW the enemy appears in the specified Y and only moves along it
	-- once he reaches the destiny, but in future versions I can set
	-- another parameter to tell if the enemy starts moving in Y Axis
	
	if v.posX <= v.AIParameter2 then
		if v.posY + v.height/2  > playerPosY then v.posY = v.posY - v.AIParameter1 end
		if v.posY + v.height/2  < playerPosY   then v.posY = v.posY + v.AIParameter1 end
	end
end
