require "var"
require "graphics"
require "input"
require "bullet"
require "player"
require "map"
require "enemy"
require "phys"

function love.load()
	love.graphics.setMode(screenWidth, screenHeight, fullScreen)
   
	love.mouse.setVisible(false)
	--love.mouse.setGrab(true)
   
	scrollSpeed = 10
	depthSpeed = 6
	
	timeCounter = 0
	oneSecond = 0
	totalObjects = 0 -- the player is not really an object like enemies or bullets are!!!
	playerHasCrashed = false
	
	playerPosX = 0
	playerPosY = 0
	playerSpeed = 3 -- from 1 to 3?
	shootReload = 5
   
	playerImage = love.graphics.newImage("data/sprites/Fighter3.png")
	playerHeight = playerImage:getHeight()
	playerWidth = playerImage:getWidth()
   
	-- this is used to estimate the distance traveled.
	-- it is a very impostant variable because it is used to spawn enemies.
	-- Also this allows better map editing because we can just set the distance the player needs
	-- to travel before spawning a certain enemy.
	-- This is a better method than time-based spawns because we can modify the speed without modifying
	-- when the enemies will be spawned
	-- And actually, distance is the same as "pixels scrolled".
	distance = 0
   
	setStageBackground("1", "1", "1", scrollSpeed, 5)
	player.setBullet("MiscBullet")
   
	loadMap("test")
end

function love.update(dt)
	
	enemy.move()
	
	-- REMOVE THIS IT'S ONLY FOR TESTING!!!!
	--enemy.linearMove()	
	
	-- Do not move player if opposite keys pressed
	if love.keyboard.isDown ("up") and not love.keyboard.isDown ("left") and not love.keyboard.isDown ("right") and not love.keyboard.isDown ("down") then
		playerPosY = playerPosY - 3 * playerSpeed
	elseif love.keyboard.isDown ("down") and not love.keyboard.isDown ("left") and not love.keyboard.isDown ("right") and not love.keyboard.isDown ("up") then
		playerPosY = playerPosY + 3 * playerSpeed
	elseif love.keyboard.isDown ("left") and not love.keyboard.isDown ("up") and not love.keyboard.isDown ("down") and not love.keyboard.isDown ("right") then
		playerPosX = playerPosX - 3 * playerSpeed
	elseif love.keyboard.isDown ("right") and not love.keyboard.isDown ("up") and not love.keyboard.isDown ("down") and not love.keyboard.isDown ("left") then
		playerPosX = playerPosX + 3 * playerSpeed
	--DIAGONALS
	elseif love.keyboard.isDown ("up") and love.keyboard.isDown ("left") then
		playerPosY = playerPosY - 3 * playerSpeed
		playerPosX = playerPosX - 3 * playerSpeed
	elseif love.keyboard.isDown ("up") and love.keyboard.isDown ("right") then
		playerPosY = playerPosY - 3 * playerSpeed
		playerPosX = playerPosX + 3 * playerSpeed
	elseif love.keyboard.isDown ("down") and love.keyboard.isDown ("left") then
		playerPosY = playerPosY + 3 * playerSpeed
		playerPosX = playerPosX - 3 * playerSpeed
	elseif love.keyboard.isDown ("down") and love.keyboard.isDown ("right") then
		playerPosY = playerPosY + 3 * playerSpeed
		playerPosX = playerPosX + 3 * playerSpeed
	end
	
	if love.keyboard.isDown("q") then 
		if shootDelay >= shootReload then
			player.shoot()
			shootDelay = 0
		end
	end
	-- Dont get out of screen bounds!!!
	if playerPosX < screenBorder then playerPosX = screenBorder end
	if playerPosX > screenWidth - playerWidth - screenBorder then playerPosX = screenWidth - playerWidth - screenBorder end
	if playerPosY < 10 then playerPosY = 10 end
	if playerPosY > screenHeight - playerHeight - screenBorder then playerPosY = screenHeight - playerHeight - screenBorder end
	
	-- Checking collisions
	gamePhysics()
	
	
	controlTime()
	
	-- will control shooting speed
	shootDelay = shootDelay + 1	
	
	distance = distance + scrollSpeed
	
	readMap()
end

function love.focus(f)
  if not f then
   -- print("LOST FOCUS")
  else
    --print("GAINED FOCUS")
  end
end

function love.quit()
  love.mouse.setVisible(true)
  love.mouse.setGrab(false)
end

function showDebugInfo()
	love.graphics.setColor (255, 255, 255, 255)
	love.graphics.print("FPS:" .. love.timer.getFPS(), 10, 10)
	love.graphics.print("Delta:" .. love.timer.getDelta(), 10, 25)
	love.graphics.print("Obj:" .. totalObjects, 10, 40)
	love.graphics.print("Pos:" .. playerPosX .. "," .. playerPosY, 10, 55)
	love.graphics.print("Time:" .. timeCounter .. ":" .. oneSecond*1000 .. " " .. tostring(onlyOneSecond), 10, 70)
	love.graphics.print("PC:" .. tostring(playerHasCrashed) , 10, 85)
	love.graphics.print("Dist:" .. distance, 10, 100)
	love.graphics.print("Map:" .. mapTitle, 10, 115)
end

function controlTime()
	if love.timer.getDelta() < 0.0365 then
		love.timer.sleep(0.0365) --50 FPS capped, approx
	end 
	
	oneSecond = oneSecond + love.timer.getDelta()
	
	if oneSecond >= 1 then
		onlyOneSecond = true 
	else 
		onlyOneSecond = false
	end
	
	if oneSecond > 1 then
		timeCounter = math.floor(timeCounter + oneSecond)
		oneSecond = 0
	end
end
